"""
DiffAlarm - 웹페이지 변화 감지기

Usage:
    diffalarm <path-to-config>
    diffalarm -h | --help

Options:
    -h --help    이 도움말을 보여줍니다.
"""

import re
import time
from dataclasses import dataclass
from pathlib import Path
from re import Pattern

import docopt
import requests
import yaml
from bs4 import BeautifulSoup as Soup
from typing_extensions import Self

from .beep import beep


@dataclass
class Page:
    url: str
    selector: str
    regex: Pattern[str]
    criteria: int
    interval: int

    @classmethod
    def from_path(cls, path: str | Path) -> "Page":
        data = yaml.safe_load(Path(path).read_text())
        data["regex"] = re.compile(data["regex"])
        return cls(**data)

    def run(self):
        while True:
            html = requests.get(self.url)
            soup = Soup(html.text, "html.parser")
            self.check(soup)
            time.sleep(self.interval)

    def check(self, soup: Soup):
        match soup.select_one(self.selector):
            case None:
                print(
                    f"CSS Selector 규칙 {self.selector}가 잘못되었거나,"
                    "해당 페이지에는 해당 요소가 없습니다."
                )
                exit(1)
            case _ as tag:
                data = "".join(re.findall(self.regex, tag.text.strip()))
                number = int(data)
                if number >= self.criteria:
                    print(f"{number}이 {self.criteria} 이상입니다.")
                    beep(sound="ping")


def main():
    assert __doc__ is not None

    path = Path(docopt.docopt(__doc__).get("<path-to-config>"))  # type: ignore
    if not path.exists():
        print(f"{path.absolute()}가 존재하지 않는 경로입니다.")
        exit(1)

    page = Page.from_path(path)
    page.run()


if __name__ == "__main__":
    main()



from typing import Literal

NUMBER_KEY = Literal[1, 2, 3, 4, 5, 6, 7]
NAMED_KEY = Literal[
    "coin", "robot_error", "error", "ping", "ready", "success", "wilhelm"
]
from beepy import beep as _beep  # type: ignore


def beep(*, sound: NUMBER_KEY | NAMED_KEY) -> None:
    _beep(sound=sound)  # type: ignore



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['diffalarm']

package_data = \
{'': ['*']}

install_requires = \
['beepy>=1.0.7,<2.0.0',
 'bs4>=0.0.1,<0.0.2',
 'docopt>=0.6.2,<0.7.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['cli_command_name = diffalarm:main']}

setup_kwargs = {
    'name': 'diffalarm',
    'version': '0.1.0',
    'description': 'alarms when web page has diff',
    'long_description': '# diffalarm\nalarms when a web pages has diff\n',
    'author': 'scarf',
    'author_email': 'greenscarf005@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/scarf005/diffalarm',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
